@@ -38,10 +38,10 @@ extern "C" {
 #define _WIN32_WINNT 0x0501
 
 #define PACKAGE "BitlBee"
-#define BITLBEE_VERSION "3.0.5"
+#define BITLBEE_VERSION "3.0.6"
 #define VERSION BITLBEE_VERSION
 #define BITLBEE_VER(a,b,c) (((a) << 16) + ((b) << 8) + (c))
-#define BITLBEE_VERSION_CODE BITLBEE_VER(3, 0, 5)
+#define BITLBEE_VERSION_CODE BITLBEE_VER(3, 0, 6)
 
 #define MAX_STRING 511
 
@@ -106,6 +106,11 @@ extern "C" {
 #undef g_main_quit
 #define g_main_quit		__PLEASE_USE_B_MAIN_QUIT__
 
+/* And now, because GLib folks think everyone loves typing ridiculously long
+   function names ... no I don't or I'd write BitlBee in Java, ffs. */
+#define g_strcasecmp g_ascii_strcasecmp
+#define g_strncasecmp g_ascii_strncasecmp
+
 #ifndef G_GNUC_MALLOC
 /* Doesn't exist in GLib <=2.4 while everything else in BitlBee should
    work with it, so let's fake this one. */
@@ -122,7 +127,7 @@ extern "C" {
 
 #define DEFAULT_AWAY "Away from computer"
 #define CONTROL_TOPIC "Welcome to the control channel. Type \2help\2 for help information."
-#define IRCD_INFO "BitlBee <http://www.bitlbee.org/>"
+#define IRCD_INFO PACKAGE " <http://www.bitlbee.org/>"
 
 #define MAX_NICK_LENGTH 24
 
