@@ -447,7 +447,18 @@ static void irc_cmd_oper_hack( irc_t *irc, char **cmd )
 	}
 	else if( irc->status & OPER_HACK_IDENTIFY )
 	{
-		char *send_cmd[] = { "identify", password, NULL };
+		char *send_cmd[] = { "identify", password, NULL, NULL };
+		irc->status &= ~OPER_HACK_IDENTIFY;
+		if( irc->status & OPER_HACK_IDENTIFY_NOLOAD )
+		{
+			send_cmd[1] = "-noload";
+			send_cmd[2] = password;
+		}
+		else if( irc->status & OPER_HACK_IDENTIFY_FORCE )
+		{
+			send_cmd[1] = "-force";
+			send_cmd[2] = password;
+		}
 		irc_send_num( irc, 491, ":Trying to identify" );
 		root_command( irc, send_cmd );
 	}
@@ -670,8 +681,8 @@ static void irc_cmd_list( irc_t *irc, char **cmd )
 
 static void irc_cmd_version( irc_t *irc, char **cmd )
 {
-	irc_send_num( irc, 351, "bitlbee-%s. %s :%s/%s ",
-	              BITLBEE_VERSION, irc->root->host, ARCH, CPU );
+	irc_send_num( irc, 351, "%s-%s. %s :%s/%s ",
+	              PACKAGE, BITLBEE_VERSION, irc->root->host, ARCH, CPU );
 }
 
 static void irc_cmd_completions( irc_t *irc, char **cmd )
