@@ -29,6 +29,9 @@
 
 #include "xmltree.h"
 
+#define g_strcasecmp g_ascii_strcasecmp
+#define g_strncasecmp g_ascii_strncasecmp
+
 static void xt_start_element( GMarkupParseContext *ctx, const gchar *element_name, const gchar **attr_names, const gchar **attr_values, gpointer data, GError **error )
 {
 	struct xt_parser *xt = data;
@@ -259,15 +262,21 @@ void xt_cleanup( struct xt_parser *xt, struct xt_node *node, int depth )
 	}
 }
 
-struct xt_node *xt_from_string( const char *in )
+struct xt_node *xt_from_string( const char *in, int len )
 {
 	struct xt_parser *parser;
-	struct xt_node *ret;
+	struct xt_node *ret = NULL;
+	
+	if( len == 0 )
+		len = strlen( in );
 	
 	parser = xt_new( NULL, NULL );
-	xt_feed( parser, in, strlen( in ) );
-	ret = parser->root;
-	parser->root = NULL;
+	xt_feed( parser, in, len );
+	if( parser->cur == NULL )
+	{
+		ret = parser->root;
+		parser->root = NULL;
+	}
 	xt_free( parser );
 	
 	return ret;
