@@ -579,7 +579,9 @@ int jabber_buddy_remove( struct im_connection *ic, char *full_jid_ )
 		    ( ( s == NULL && bud->resource == NULL ) ||
 		      ( bud->resource && s && strcmp( bud->resource, s + 1 ) == 0 ) ) )
 		{
-			return jabber_buddy_remove_bare( ic, full_jid );
+			int st = jabber_buddy_remove_bare( ic, full_jid );
+			g_free( full_jid );
+			return st;
 		}
 		else if( s == NULL || bud->resource == NULL )
 		{
@@ -673,6 +675,8 @@ static gboolean jabber_buddy_remove_all_cb( gpointer key, gpointer value, gpoint
 	struct jabber_buddy *bud, *next;
 	
 	bud = value;
+	if( bud->bare_jid != bud->full_jid )
+		g_free( bud->bare_jid );
 	while( bud )
 	{
 		next = bud->next;
@@ -758,3 +762,21 @@ void jabber_error_free( struct jabber_error *err )
 {
 	g_free( err );
 }
+
+gboolean jabber_set_me( struct im_connection *ic, const char *me )
+{
+	struct jabber_data *jd = ic->proto_data;
+	
+	if( strchr( me, '@' ) == NULL )
+		return FALSE;
+	
+	g_free( jd->username );
+	g_free( jd->me );
+	
+	jd->me = jabber_normalize( me );
+	jd->server = strchr( jd->me, '@' );
+	jd->username = g_strndup( jd->me, jd->server - jd->me );
+	jd->server ++;
+	
+	return TRUE;
+}
