@@ -241,16 +241,13 @@ static void twitter_init(account_t * acc)
 {
 	set_t *s;
 	char *def_url;
-	char *def_oauth;
 	char *def_tul;
 
 	if (strcmp(acc->prpl->name, "twitter") == 0) {
 		def_url = TWITTER_API_URL;
-		def_oauth = "true";
 		def_tul = "20";
 	} else {		/* if( strcmp( acc->prpl->name, "identica" ) == 0 ) */
 		def_url = IDENTICA_API_URL;
-		def_oauth = "true";
 		def_tul = "0";
 	}
 
@@ -273,9 +270,9 @@ static void twitter_init(account_t * acc)
 	s = set_add(&acc->set, "mode", "chat", set_eval_mode, acc);
 	s->flags |= ACC_SET_OFFLINE_ONLY;
 
-	s = set_add(&acc->set, "oauth", def_oauth, set_eval_oauth, acc);
+	s = set_add(&acc->set, "oauth", "true", set_eval_oauth, acc);
 
-	s = set_add(&acc->set, "show_ids", "false", set_eval_bool, acc);
+	s = set_add(&acc->set, "show_ids", "true", set_eval_bool, acc);
 	s->flags |= ACC_SET_OFFLINE_ONLY;
 
 	s = set_add(&acc->set, "show_old_mentions", "20", set_eval_int, acc);
@@ -492,6 +489,29 @@ static void twitter_buddy_data_free(struct bee_user *bu)
 	g_free(bu->data);
 }
 
+/** Convert the given bitlbee tweet ID, bitlbee username, or twitter tweet ID
+ *  into a twitter tweet ID.
+ *
+ *  Returns 0 if the user provides garbage.
+ */
+static guint64 twitter_message_id_from_command_arg(struct im_connection *ic, struct twitter_data *td, char *arg) {
+	struct twitter_user_data *tud;
+	bee_user_t *bu;
+	guint64 id = 0;
+	if (g_str_has_prefix(arg, "#") &&
+		sscanf(arg + 1, "%" G_GUINT64_FORMAT, &id) == 1) {
+		if (id < TWITTER_LOG_LENGTH && td->log)
+			id = td->log[id].id;
+	} else if ((bu = bee_user_by_handle(ic->bee, ic, arg)) &&
+		(tud = bu->data) && tud->last_id)
+		id = tud->last_id;
+	else if (sscanf(arg, "%" G_GUINT64_FORMAT, &id) == 1){
+		if (id < TWITTER_LOG_LENGTH && td->log)
+			id = td->log[id].id;
+	}
+	return id;
+}
+
 static void twitter_handle_command(struct im_connection *ic, char *message)
 {
 	struct twitter_data *td = ic->proto_data;
@@ -519,6 +539,15 @@ static void twitter_handle_command(struct im_connection *ic, char *message)
 		} else
 			twitter_msg(ic, "Could not undo last action");
 
+		g_free(cmds);
+		return;
+	} else if (g_strcasecmp(cmd[0], "favourite") == 0 && cmd[1]) {
+		guint64 id;
+		if ((id = twitter_message_id_from_command_arg(ic, td, cmd[1]))) {
+			twitter_favourite_tweet(ic, id);
+		} else {
+			twitter_msg(ic, "Please provide a message ID or username.");
+		}
 		g_free(cmds);
 		return;
 	} else if (g_strcasecmp(cmd[0], "follow") == 0 && cmd[1]) {
@@ -529,22 +558,26 @@ static void twitter_handle_command(struct im_connection *ic, char *message)
 		twitter_remove_buddy(ic, cmd[1], NULL);
 		g_free(cmds);
 		return;
-	} else if (g_strcasecmp(cmd[0], "rt") == 0 && cmd[1]) {
-		struct twitter_user_data *tud;
-		bee_user_t *bu;
+	} else if ((g_strcasecmp(cmd[0], "report") == 0 ||
+	            g_strcasecmp(cmd[0], "spam") == 0) && cmd[1]) {
+		char * screen_name;
 		guint64 id;
-
+		screen_name = cmd[1];
+		/* Report nominally works on users but look up the user who
+		   posted the given ID if the user wants to do it that way */
 		if (g_str_has_prefix(cmd[1], "#") &&
 		    sscanf(cmd[1] + 1, "%" G_GUINT64_FORMAT, &id) == 1) {
-			if (id < TWITTER_LOG_LENGTH && td->log)
-				id = td->log[id].id;
-		} else if ((bu = bee_user_by_handle(ic->bee, ic, cmd[1])) &&
-		    (tud = bu->data) && tud->last_id)
-			id = tud->last_id;
-		else if (sscanf(cmd[1], "%" G_GUINT64_FORMAT, &id) == 1){
-			if (id < TWITTER_LOG_LENGTH && td->log)
-				id = td->log[id].id;
+			if (id < TWITTER_LOG_LENGTH && td->log) {
+				if (g_slist_find(ic->bee->users, td->log[id].bu)) {
+					screen_name = td->log[id].bu->handle;
+				}
+			}
 		}
+		twitter_report_spam(ic, screen_name);
+		g_free(cmds);
+		return;
+	} else if (g_strcasecmp(cmd[0], "rt") == 0 && cmd[1]) {
+		guint64 id = twitter_message_id_from_command_arg(ic, td, cmd[1]);
 
 		td->last_status_id = 0;
 		if (id)
@@ -583,6 +616,7 @@ static void twitter_handle_command(struct im_connection *ic, char *message)
 		if (!id || !bu) {
 			twitter_msg(ic, "User `%s' does not exist or didn't "
 				    "post any statuses recently", cmd[1]);
+			g_free(cmds);
 			return;
 		}
 		message = new = g_strdup_printf("@%s %s", bu->handle, message + (cmd[2] - cmd[0]));
