@@ -4,7 +4,7 @@
 *  Simple module to facilitate twitter functionality.                       *
 *                                                                           *
 *  Copyright 2009-2010 Geert Mulders <g.c.w.m.mulders@gmail.com>            *
-*  Copyright 2010-2011 Wilmer van der Gaast <wilmer@gaast.net>              *
+*  Copyright 2010-2012 Wilmer van der Gaast <wilmer@gaast.net>              *
 *                                                                           *
 *  This library is free software; you can redistribute it and/or            *
 *  modify it under the terms of the GNU Lesser General Public               *
@@ -167,28 +167,72 @@ static void twitter_add_buddy(struct im_connection *ic, char *name, const char *
 char *twitter_parse_error(struct http_request *req)
 {
 	static char *ret = NULL;
-	struct xt_parser *xp = NULL;
-	struct xt_node *node, *err;
+	struct xt_node *root, *node, *err;
 
 	g_free(ret);
 	ret = NULL;
 
 	if (req->body_size > 0) {
-		xp = xt_new(NULL, NULL);
-		xt_feed(xp, req->reply_body, req->body_size);
+		root = xt_from_string(req->reply_body, req->body_size);
 		
-		for (node = xp->root; node; node = node->next)
+		for (node = root; node; node = node->next)
 			if ((err = xt_find_node(node->children, "error")) && err->text_len > 0) {
 				ret = g_strdup_printf("%s (%s)", req->status_string, err->text);
 				break;
 			}
 
-		xt_free(xp);
+		xt_free_node(root);
 	}
 
 	return ret ? ret : req->status_string;
 }
 
+static struct xt_node *twitter_parse_response(struct im_connection *ic, struct http_request *req)
+{
+	gboolean logging_in = !(ic->flags & OPT_LOGGED_IN);
+	gboolean periodic;
+	struct twitter_data *td = ic->proto_data;
+	struct xt_node *ret;
+	char path[64] = "", *s;
+	
+	if ((s = strchr(req->request, ' '))) {
+		path[sizeof(path)-1] = '\0';
+		strncpy(path, s + 1, sizeof(path) - 1);
+		if ((s = strchr(path, '?')) || (s = strchr(path, ' ')))
+			*s = '\0';
+	}
+	
+	/* Kinda nasty. :-( Trying to suppress error messages, but only
+	   for periodic (i.e. mentions/timeline) queries. */
+	periodic = strstr(path, "timeline") || strstr(path, "mentions");
+	
+	if (req->status_code == 401 && logging_in) {
+		/* IIRC Twitter once had an outage where they were randomly
+		   throwing 401s so I'll keep treating this one as fatal
+		   only during login. */
+		imcb_error(ic, "Authentication failure");
+		imc_logout(ic, FALSE);
+		return NULL;
+	} else if (req->status_code != 200) {
+		// It didn't go well, output the error and return.
+		if (!periodic || logging_in || ++td->http_fails >= 5)
+			imcb_error(ic, "Could not retrieve %s: %s",
+				   path, twitter_parse_error(req));
+		
+		if (logging_in)
+			imc_logout(ic, TRUE);
+		return NULL;
+	} else {
+		td->http_fails = 0;
+	}
+
+	if ((ret = xt_from_string(req->reply_body, req->body_size)) == NULL) {
+		imcb_error(ic, "Could not retrieve %s: %s",
+			   path, "XML parse error");
+	}
+	return ret;
+}
+
 static void twitter_http_get_friends_ids(struct http_request *req);
 
 /**
@@ -255,7 +299,7 @@ static void twitter_get_users_lookup(struct im_connection *ic);
 static void twitter_http_get_friends_ids(struct http_request *req)
 {
 	struct im_connection *ic;
-	struct xt_parser *parser;
+	struct xt_node *parsed;
 	struct twitter_xml_list *txl;
 	struct twitter_data *td;
 
@@ -267,22 +311,6 @@ static void twitter_http_get_friends_ids(struct http_request *req)
 
 	td = ic->proto_data;
 
-	// Check if the HTTP request went well. More strict checks as this is
-	// the first request we do in a session.
-	if (req->status_code == 401) {
-		imcb_error(ic, "Authentication failure");
-		imc_logout(ic, FALSE);
-		return;
-	} else if (req->status_code != 200) {
-		// It didn't go well, output the error and return.
-		imcb_error(ic, "Could not retrieve %s: %s",
-			   TWITTER_FRIENDS_IDS_URL, twitter_parse_error(req));
-		imc_logout(ic, TRUE);
-		return;
-	} else {
-		td->http_fails = 0;
-	}
-
 	/* Create the room now that we "logged in". */
 	if (!td->timeline_gc && g_strcasecmp(set_getstr(&ic->acc->set, "mode"), "chat") == 0)
 		twitter_groupchat_init(ic);
@@ -291,10 +319,10 @@ static void twitter_http_get_friends_ids(struct http_request *req)
 	txl->list = td->follow_ids;
 
 	// Parse the data.
-	parser = xt_new(NULL, txl);
-	xt_feed(parser, req->reply_body, req->body_size);
-	twitter_xt_get_friends_id_list(parser->root, txl);
-	xt_free(parser);
+	if (!(parsed = twitter_parse_response(ic, req)))
+		return;
+	twitter_xt_get_friends_id_list(parsed, txl);
+	xt_free_node(parsed);
 
 	td->follow_ids = txl->list;
 	if (txl->next_cursor)
@@ -350,8 +378,7 @@ static void twitter_get_users_lookup(struct im_connection *ic)
 static void twitter_http_get_users_lookup(struct http_request *req)
 {
 	struct im_connection *ic = req->data;
-	struct twitter_data *td;
-	struct xt_parser *parser;
+	struct xt_node *parsed;
 	struct twitter_xml_list *txl;
 	GSList *l = NULL;
 	struct twitter_xml_user *user;
@@ -360,28 +387,14 @@ static void twitter_http_get_users_lookup(struct http_request *req)
 	if (!g_slist_find(twitter_connections, ic))
 		return;
 
-	td = ic->proto_data;
-
-	if (req->status_code != 200) {
-		// It didn't go well, output the error and return.
-		imcb_error(ic, "Could not retrieve %s: %s",
-			   TWITTER_USERS_LOOKUP_URL, twitter_parse_error(req));
-		imc_logout(ic, TRUE);
-		return;
-	} else {
-		td->http_fails = 0;
-	}
-
 	txl = g_new0(struct twitter_xml_list, 1);
 	txl->list = NULL;
 
-	// Parse the data.
-	parser = xt_new(NULL, txl);
-	xt_feed(parser, req->reply_body, req->body_size);
-
 	// Get the user list from the parsed xml feed.
-	twitter_xt_get_users(parser->root, txl);
-	xt_free(parser);
+	if (!(parsed = twitter_parse_response(ic, req)))
+		return;
+	twitter_xt_get_users(parsed, txl);
+	xt_free_node(parsed);
 
 	// Add the users as buddies.
 	for (l = txl->list; l; l = g_slist_next(l)) {
@@ -500,23 +513,30 @@ static xt_status twitter_xt_get_status(struct xt_node *node, struct twitter_xml_
 	} else {
 		struct xt_node *urls, *url;
 		
-		urls = xt_find_path(node, "entities/urls");
-		for (url = urls ? urls->children : NULL; url; url = url->next) {
-			/* "short" is a reserved word. :-P */
-			struct xt_node *kort = xt_find_node(url->children, "url");
-			struct xt_node *disp = xt_find_node(url->children, "display_url");
-			char *pos, *new;
-			
-			if (!kort || !kort->text || !disp || !disp->text ||
-			    !(pos = strstr(txs->text, kort->text)))
+		urls = xt_find_path(node, "entities");
+		if (urls != NULL)
+			urls = urls->children;
+		for (; urls; urls = urls->next) {
+			if (strcmp(urls->name, "urls") != 0 && strcmp(urls->name, "media") != 0)
 				continue;
 			
-			*pos = '\0';
-			new = g_strdup_printf("%s%s &lt;%s&gt;%s", txs->text, kort->text,
-			                      disp->text, pos + strlen(kort->text));
-			
-			g_free(txs->text);
-			txs->text = new;
+			for (url = urls ? urls->children : NULL; url; url = url->next) {
+				/* "short" is a reserved word. :-P */
+				struct xt_node *kort = xt_find_node(url->children, "url");
+				struct xt_node *disp = xt_find_node(url->children, "display_url");
+				char *pos, *new;
+				
+				if (!kort || !kort->text || !disp || !disp->text ||
+				    !(pos = strstr(txs->text, kort->text)))
+					continue;
+				
+				*pos = '\0';
+				new = g_strdup_printf("%s%s &lt;%s&gt;%s", txs->text, kort->text,
+				                      disp->text, pos + strlen(kort->text));
+				
+				g_free(txs->text);
+				txs->text = new;
+			}
 		}
 	}
 
@@ -731,7 +751,10 @@ void twitter_get_timeline(struct im_connection *ic, gint64 next_cursor)
 	gboolean include_mentions = set_getbool(&ic->acc->set, "fetch_mentions");
 
 	if (td->flags & TWITTER_DOING_TIMELINE) {
-		return;
+		if (++td->http_fails >= 5) {
+			imcb_error(ic, "Fetch timeout (%d)", td->flags);
+			imc_logout(ic, TRUE);
+		}
 	}
 
 	td->flags |= TWITTER_DOING_TIMELINE;
@@ -780,6 +803,9 @@ void twitter_flush_timeline(struct im_connection *ic)
 			output = g_slist_insert_sorted(output, l->data, twitter_compare_elements);
 		}
 	}
+	
+	if (!(ic->flags & OPT_LOGGED_IN))
+		imcb_connected(ic);
 
 	// See if the user wants to see the messages in a groupchat window or as private messages.
 	if (g_strcasecmp(set_getstr(&ic->acc->set, "mode"), "chat") == 0)
@@ -878,7 +904,7 @@ static void twitter_http_get_home_timeline(struct http_request *req)
 {
 	struct im_connection *ic = req->data;
 	struct twitter_data *td;
-	struct xt_parser *parser;
+	struct xt_node *parsed;
 	struct twitter_xml_list *txl;
 
 	// Check if the connection is still active.
@@ -887,29 +913,14 @@ static void twitter_http_get_home_timeline(struct http_request *req)
 
 	td = ic->proto_data;
 
-	// Check if the HTTP request went well.
-	if (req->status_code == 200) {
-		td->http_fails = 0;
-		if (!(ic->flags & OPT_LOGGED_IN))
-			imcb_connected(ic);
-	} else {
-		// It didn't go well, output the error and return.
-		if (++td->http_fails >= 5)
-			imcb_error(ic, "Could not retrieve %s: %s",
-				   TWITTER_HOME_TIMELINE_URL, twitter_parse_error(req));
-
-		goto end;
-	}
-
 	txl = g_new0(struct twitter_xml_list, 1);
 	txl->list = NULL;
 
-	// Parse the data.
-	parser = xt_new(NULL, txl);
-	xt_feed(parser, req->reply_body, req->body_size);
 	// The root <statuses> node should hold the list of statuses <status>
-	twitter_xt_get_status_list(ic, parser->root, txl);
-	xt_free(parser);
+	if (!(parsed = twitter_parse_response(ic, req)))
+		goto end;
+	twitter_xt_get_status_list(ic, parsed, txl);
+	xt_free_node(parsed);
 
 	td->home_timeline_obj = txl;
 
@@ -926,7 +937,7 @@ static void twitter_http_get_mentions(struct http_request *req)
 {
 	struct im_connection *ic = req->data;
 	struct twitter_data *td;
-	struct xt_parser *parser;
+	struct xt_node *parsed;
 	struct twitter_xml_list *txl;
 
 	// Check if the connection is still active.
@@ -935,29 +946,14 @@ static void twitter_http_get_mentions(struct http_request *req)
 
 	td = ic->proto_data;
 
-	// Check if the HTTP request went well.
-	if (req->status_code == 200) {
-		td->http_fails = 0;
-		if (!(ic->flags & OPT_LOGGED_IN))
-			imcb_connected(ic);
-	} else {
-		// It didn't go well, output the error and return.
-		if (++td->http_fails >= 5)
-			imcb_error(ic, "Could not retrieve %s: %s",
-				   TWITTER_MENTIONS_URL, twitter_parse_error(req));
-
-		goto end;
-	}
-
 	txl = g_new0(struct twitter_xml_list, 1);
 	txl->list = NULL;
 
-	// Parse the data.
-	parser = xt_new(NULL, txl);
-	xt_feed(parser, req->reply_body, req->body_size);
 	// The root <statuses> node should hold the list of statuses <status>
-	twitter_xt_get_status_list(ic, parser->root, txl);
-	xt_free(parser);
+	if (!(parsed = twitter_parse_response(ic, req)))
+		goto end;
+	twitter_xt_get_status_list(ic, parsed, txl);
+	xt_free_node(parsed);
 
 	td->mentions_obj = txl;
 
@@ -975,6 +971,7 @@ static void twitter_http_post(struct http_request *req)
 {
 	struct im_connection *ic = req->data;
 	struct twitter_data *td;
+	struct xt_node *parsed, *node;
 
 	// Check if the connection is still active.
 	if (!g_slist_find(twitter_connections, ic))
@@ -983,26 +980,12 @@ static void twitter_http_post(struct http_request *req)
 	td = ic->proto_data;
 	td->last_status_id = 0;
 
-	// Check if the HTTP request went well.
-	if (req->status_code != 200) {
-		// It didn't go well, output the error and return.
-		imcb_error(ic, "HTTP error: %s", twitter_parse_error(req));
+	if (!(parsed = twitter_parse_response(ic, req)))
 		return;
-	}
-
-	if (req->body_size > 0) {
-		struct xt_parser *xp = NULL;
-		struct xt_node *node;
-
-		xp = xt_new(NULL, NULL);
-		xt_feed(xp, req->reply_body, req->body_size);
-
-		if ((node = xt_find_node(xp->root, "status")) &&
-		    (node = xt_find_node(node->children, "id")) && node->text)
-			td->last_status_id = g_ascii_strtoull(node->text, NULL, 10);
-
-		xt_free(xp);
-	}
+	
+	if ((node = xt_find_node(parsed, "status")) &&
+	    (node = xt_find_node(node->children, "id")) && node->text)
+		td->last_status_id = g_ascii_strtoull(node->text, NULL, 10);
 }
 
 /**
@@ -1061,3 +1044,29 @@ void twitter_status_retweet(struct im_connection *ic, guint64 id)
 	twitter_http(ic, url, twitter_http_post, ic, 1, NULL, 0);
 	g_free(url);
 }
+
+/**
+ * Report a user for sending spam.
+ */
+void twitter_report_spam(struct im_connection *ic, char *screen_name)
+{
+	char *args[2] = {
+		"screen_name",
+		NULL,
+	};
+	args[1] = screen_name;
+	twitter_http(ic, TWITTER_REPORT_SPAM_URL, twitter_http_post,
+	             ic, 1, args, 2);
+}
+
+/**
+ * Favourite a tweet.
+ */
+void twitter_favourite_tweet(struct im_connection *ic, guint64 id)
+{
+	char *url;
+	url = g_strdup_printf("%s%llu%s", TWITTER_FAVORITE_CREATE_URL,
+	                      (unsigned long long) id, ".xml");
+	twitter_http(ic, url, twitter_http_post, ic, 1, NULL, 0);
+	g_free(url);
+}
