@@ -48,6 +48,7 @@ typedef enum
 	SET_NULL_OK = 0x0100,
 	SET_HIDDEN = 0x0200,
 	SET_PASSWORD = 0x0400,
+	SET_HIDDEN_DEFAULT = 0x0800,
 } set_flags_t;
 
 typedef struct set
@@ -75,6 +76,8 @@ typedef struct set
 	struct set *next;
 } set_t;
 
+#define set_value( set ) ((set)->value) ? ((set)->value) : ((set)->def)
+
 /* Should be pretty clear. */
 set_t *set_add( set_t **head, const char *key, const char *def, set_eval eval, void *data );
 
@@ -97,6 +100,9 @@ int set_setint( set_t **head, const char *key, int value );
 void set_del( set_t **head, const char *key );
 int set_reset( set_t **head, const char *key );
 
+/* returns true if a setting shall be shown to the user */
+int set_isvisible( set_t *set );
+
 /* Two very useful generic evaluators. */
 char *set_eval_int( set_t *set, char *value );
 char *set_eval_bool( set_t *set, char *value );
@@ -106,6 +112,6 @@ char *set_eval_list( set_t *set, char *value );
 
 /* Some not very generic evaluators that really shouldn't be here... */
 char *set_eval_to_char( set_t *set, char *value );
-char *set_eval_ops( set_t *set, char *value );
+char *set_eval_oauth( set_t *set, char *value );
 
 #endif /* __SET_H__ */
